;**************************************************************
;
;   BLOQUEADOR TEMPORIZADO C/ SENHA POR 2 BOTES
;   UTILIZA UM OUTRO BOTO PARA INICIAR O TEMPO DE DESLIGAMENTO
;
;   MICROCONTROLADOR: PIC12F629/629
;
;	AUTOR:  CLUDIO  LRIOS
;
;   DATA:   25/04/2017
;
;  OBJETIVO: 
;  APERTANDO UM BOTO INICIAMOS UM TEMPO DE DESLIGAMENTO EM TORNO DE 2 MINUTOS
;  APS ESTE TEMPO O RELE DESLIGA FAZENDO PARAR O MOTOR DO VEICULO OU MOTO
;  (CORTA A BOBINA OU A BOMBA DE COMBUSTIVEL, OU AINDA CURTO-CIRCUITA O PLATINADO).
;  PARA FUNCIONAR APS ACIONAR O BOTO SER NECESSRIO ENTRAR COM UMA SENHA USANDO
;  OS DOIS BOTES. A SENHA  DE 8 DIGITOS. UM BOTO SER '1' E O OUTRO '0'.
;  SENHA INICIAL= 11101000
;  PERMITE TROCAR A SENHA POR OUTRA. PARA ISTO:
;  1)DIGITE A SENHA DE TROCA (10001000),GERANDO UM APITO
;  2)DIGITE A SENHA ATUAL (11101000),GERANDO 2 APITOS.
;  3)DIGITE A NOVA SENHA (XXXXXXXX),GERANDO 3 APITOS.
;  4)CONFIRME DIGITANDO A NOVA SENHA (XXXXXXXX),GERANDO APITO LONGO.
;  APS 3 ERROS, TRAVA A DIGITAO POR 3 MINUTOS, EMITINDO UM APITO CADENCIADO.
;
;   USO DIDTICO APENAS.
;
;   BLOG 'LARIOS.TECNOLOGIA.WS'
;
;=====================================================================================================
;ESCOLHA AQUI O MODELO DO MICROCONTROLADOR (COMENTE O NO USADO)
;#DEFINE  PIC12F629
#DEFINE  PIC675
;=====================================================================================================

;=====================================================================================================
; EQUATES
NUMERO_DE_ERROS 	EQU       0X03          ;QUANTIDADE DE VEZES QUE SE PODE DIGITAR SENHA ERRADA
SENHA_INIT    		EQU		  B'11101000'   ;SENHA INICIAL PR-FIXADA 
SENHA_TROCA         EQU  	  B'10001000'   ;SENHA COMANDO PARA TROCA DE SENHA
END_SIT             EQU       .5            ;ENDEREO DA EEPROM COM SITUAO INICIAL DO RELE
END_BLOQUEADO       EQU       .6            ;ENDEREO SE FOI TRAVADO POR SENHA ERRADA
END_DISP            EQU       .7            ;ENDEREO SE FOI DISPARADO
END_TIME            EQU       .8            ;ENDEREO CONTADOR HIGH DO TEMPO RESTANTE PARA BLOQUEIO
END_TECLADO_TRAVADO EQU       .9            ;SALVA TRAVAMENTO DO TECLADO POR VRIAS TENTATIVAS
T_APERTADO          EQU       0X30          ;TEMPO SEM TECLAR MAXIMO (0X0A= 30SEG).
TEMPO_S_TEC         EQU       0x50          ;TEMPO SEM TECLAR NADA (0X50=10 SEG)
T_P                 EQU       .2            ;TEMPO DO PULSO ON
TEMPO_PULSANDO      EQU       .60           ;TEMPO PULSANDO A SAIDA
T_TRAVADO           EQU       .180		    ;TEMPO TRAVADO APOS 3 ERROS (180= 3 MINUTOS)
VALOR_X2            EQU       .7           ;TEMPO PARA INICIAR O BLOQUEIO PARA 2 MINUTOS
TEMPO_SIMULACAO     EQU       .2           ;FUNCIONA POR 2 SEGUNDOS E DESLIGA MOTOR
FREQ_PISCADA        EQU       .16          ;DETERMINA A FREQUENCIA DA PISCADA DO LED EM MODO BLOQUEADO
DEBOUNCE            EQU       0XFF
;=====================================================================================================
	IFDEF PIC629
#INCLUDE<P12F629.INC> ;    ARQUIVO PADRAO
	LIST P=12f629 , R=DEC
	ENDIF	

	IFDEF PIC675
#INCLUDE<P12F675.INC> ;    ARQUIVO PADRAO
	LIST P=12f675 , R=DEC
	ENDIF
	__CONFIG	_BODEN_ON & _CP_OFF & _PWRTE_ON & _WDT_OFF  & _MCLRE_OFF  &  _INTRC_OSC_NOCLKOUT

	ERRORLEVEL      -302     			;ELIMINA MENSAGENS DE BANCO ERRADO
;---------------------------------------------------------------------------------------------
; DEFINES
#DEFINE T0IE_                INTCON,5
#DEFINE  T0IF_               INTCON,2
#DEFINE  GIE_                INTCON,7
#DEFINE	 BANK0	             BCF	STATUS,RP0 	;SETA BANK0 DE MEMORIA
#DEFINE	 BANK1	             BSF	STATUS,RP0	;SETA BANK1
#DEFINE  SOM                 GPIO,2           ;PINO 5  - SOM
#DEFINE  RELE                GPIO,4           ;PINO 3 - RELE DA FECHADURA -DIRETO
#DEFINE  SW                  GPIO,3           ;PINO 4 - ATIVA TEMPORIZAO 
#DEFINE  LED                 GPIO,5           ;PINO 2 - LED INDICADOR DA SITUAO DO BLOQUEADOR
#DEFINE  FLAG_DIGITOU        FLAGS,0          ;FLAG DE CONTROLE
#DEFINE  FLAG_PISCA          FLAGS,1          ;FLAG AUX
#DEFINE  BOTAO_0             GPIO,0           ;PINO 7  BOTO '0'
#DEFINE  BOTAO_1             GPIO,1           ;PINO 6  BOTO '1'

;--------------------------------------------------------------------------------------------
;  OPO DO USURIO:
#DEFINE TRAVA_ENTRADA_SENHA ; APS 3 TENTATIVAS ERRADA DE DIGITAR A SENHA TRAVA APITANDO POR UM TEMPO
;#DEFINE  AVISO_DISPARADO_5_BIPS ;PRODUZ 5 BIPS AO LIGAR CONTATO COM SISTEMA DISPARADO
;#DEFINE  LIGA_MOMENTANEO        ;AO RELIGAR A CHAVE DE CONTATO LIGA RELE POR 2 SEGUNDOS APENAS

;--------------------------------------------------------------------------------------------

;RAM
		CBLOCK		0X20		;ENDERECO INICIAL DE MEMORIA
    TEMP,TEMP1,TEMP2,FLAGS,AUX,AUX1,AUX2,CNT,CNT1,CNT2,RER,TMP,R0,R1,R2,XLC
    STATUSTEMP,WTWMP,BITSENHA,TIMER1,Z1,Z0,X1,X2,X3,Y1,U1,U2
       ENDC
;============================================================================================
;                                  RESET
;============================================================================================
		ORG 0X00	
 		GOTO INICIO
;============================================================================================
;                                  INTERRUPES
;============================================================================================
        ORG 0X04
        BCF  T0IF_
        DECFSZ  X1,F
        GOTO   P5
        DECFSZ  X2,F
        GOTO   P5
        BCF   T0IE_      ;ALCANOU O TEMPO: DESLIGA INTERRUPO DO TMR0
        BCF     RELE     ;CORTA MOTOR DESLIGANDO REL

        MOVLW   .0       ;SITUAO: RELE DESLIGADO  
        MOVWF   TMP
        MOVLW   END_SIT
        CALL    EEWRITE  ;

        MOVLW   .1
        MOVWF   TMP
        MOVLW   END_BLOQUEADO
        CALL    EEWRITE       
        MOVLW   VALOR_X2
        MOVWF   TMP
        MOVLW   END_TIME ;
        CALL    EEWRITE  ;RECUPERA TEMPO DE DESLIGAMENTO
        MOVWF   X2
        CLRF    X1
        CALL   APITO_C        
P5		RETFIE 
;============================================================================================
;                                  INICIO DO PROGRAMA
;============================================================================================

INICIO

		MOVLW  	.7		;DESLIGA COMPARADORES
        MOVWF  	CMCON
		BANK1
		MOVLW  	0X87   ;TMR0 DIVIDINDO POR 256 DESLIGA PULLUP
		MOVWF  	OPTION_REG ;
 
        IFDEF  	PIC675
        MOVLW  	.0  
		MOVWF  	ANSEL  ;LIBERA PORTAS DO AD
        ENDIF
        MOVLW  	B'001011'
        MOVWF  	TRISIO    ;SENTIDO DAS PORTAS
        MOVLW  	B'000011'
		BANK0
 		MOVLW  	NUMERO_DE_ERROS  ;CARREGA CONTADOR DE MAX. DE SENHAS ERRADAS
		MOVWF 	RER 
	    CLRF    FLAGS   ;RESETA TODOS OS FLAGS
        CLRF 	GPIO    ;DESLIGA TODAS AS SADAS
        BCF     T0IE_   ;DESLIGA INTERRUPO DO TIMER 0 (TEMPO DE DESLIGAMENTO)
        MOVLW   .2
        CALL    DELAYM  ;ATRASO PARA ELIMINAR FALSO DISPARO AO LIGAR/DESLIGAR CHAVE DE CONTATO
        BSF     GIE_    ;LIGA CHAVE GERAL DAS INTERRUPES
       
        MOVLW   END_TECLADO_TRAVADO  ;TESTA SE TECLADO FOI TRAVADO POR NMERO MXIMO DE TENTATIVAS
        CALL    EEREAD
        BTFSS   STATUS,Z
        GOTO    TECLADO_TRAVADO  ;FOI TRAVADO: VAI TERMINAR TEMPO DE ESPERA PROGRAMADO

        MOVLW   END_TIME ;PEGA VALOR DE X2 DA EEPROM
        CALL    EEREAD
        MOVWF   X2
        CLRF    X1
        MOVLW   END_BLOQUEADO ;TESTA SE J ESTAVA BLOQUEADO POR TEMPO
        CALL    EEREAD
        BTFSS   STATUS,Z
        GOTO    CONT_BLOQUEADO  ;ESTAVA: SEGUE COM ROTINA DE BLOQUEIO


        MOVLW   END_SIT 
        CALL    EEREAD     ;LE NA EEPROM A SITUAO DO RELE ATUAL
        BTFSS   STATUS,Z
        BSF   	RELE 	;LIGA REL PARA PERMITIR FUNCIONAMENTO (1)
  

        MOVLW   END_DISP ; TESTA SE ESTAVA DISPARADO
        CALL    EEREAD
        BTFSC   STATUS,Z
        GOTO    BAU1  ;NO ESTAVA DISPARADO PELO BOTO
  

      ;DECREMENTA 1 (16 SEGUNDOS) PARA CADA OPERAO DE LIGA/DESLIGA DA CHAVE DE CONTATO APS DISPARADO
        MOVLW   END_TIME
        CALL    EEREAD
        MOVWF   TMP
        XORLW   .1         ;LIMITA A 1
        BTFSC   STATUS,Z
        GOTO    BLOQ4
        DECF    TMP,F       ;DECREMENTA 1
        MOVLW   END_TIME
        CALL    EEWRITE     ;SALVA VALOR SUBTRAIDO 1
        GOTO    XV5
;---------------------------------------------------------------------
;  BLOQUEIO POR LIGAR/DESLIGAR CONTATO POR MAIS DE 7 VEZES COM
 ; EM  MODO DISPARADO
BLOQ4
        BCF     T0IE_      ;ALCANOU O TEMPO: DESLIGA INTERRUPO DO TMR0
        BCF     RELE     ;CORTA MOTOR DESLIGANDO REL

        MOVLW   .0       ;SITUAO: RELE DESLIGADO  
        MOVWF   TMP
        MOVLW   END_SIT
        CALL    EEWRITE  ;

        MOVLW   .1
        MOVWF   TMP
        MOVLW   END_BLOQUEADO
        CALL    EEWRITE       
        MOVLW   VALOR_X2
        MOVWF   TMP
        MOVLW   END_TIME ;
        CALL    EEWRITE  ;RECUPERA TEMPO DE DESLIGAMENTO
        MOVWF   X2
        CLRF    X1
        CALL   APITO_C  
        GOTO   MAIN
;----------------------------------------OPO-------------------------------------------
XV5 

      IFDEF   AVISO_DISPARADO_5_BIPS
        MOVLW    .5
        MOVWF    X3
LOOP7Y
        CALL      BIP  
        MOVLW     .5      ; 5 BIPS PARA AVISAR QUE EST DISPARADO (AO LIGAR CONTATO)
        CALL      DELAYM
        DECFSZ   X3,F
        GOTO     LOOP7Y
      ENDIF
;------------------------------------------------------------------------------------------
        GOTO    F_DISP   ;VAI PARA ROTINA PARA MANTER DISPARADO


BAU1    BSF     LED
;============================================================================================
;                                  ROTINA PRINCIPAL
;============================================================================================

MAIN

        MOVLW   END_BLOQUEADO
        CALL    EEREAD
        BTFSS   STATUS,Z
        BSF     FLAG_PISCA  ;PISCA O LED SE BLOQUEADO
        BTFSC   STATUS,Z
        BCF     FLAG_PISCA
        BTFSS   FLAG_PISCA
        GOTO    ESC7
        DECFSZ  U1,F
        GOTO    ESC7
        MOVLW   FREQ_PISCADA ;DETERMINA A FREQUNCIA DA PISCADA DO LED
        MOVWF   U1
        DECFSZ  U2,F
        GOTO    ESC7
        BTFSS   LED   ;LIGA LED
        GOTO    LIGA
        BCF     LED  ;DESLIGA LED
        GOTO    ESC7 ;PULA 
LIGA    BSF     LED  ;LIGA LED
ESC7
        BTFSC   SW   ;TESTA INTERRUPTOR DE DISPARO DO TEMPO DE DESLIGAMENTO
        GOTO    PULAT1

F_DISP
        BTFSS   T0IE_  ;SE INTERRUPO DO TIMER_0 ESTIVER DESABILITADA
        GOTO    FOI3   ;DESLIGADA: VAI LIGAR,BIPAR E SALVAR NA EEPROM
VOLTA3
        BSF     T0IE_  ; CONTA TEMPO NA INTERRUPO DO TIMER 0
        GOTO   PULAT1
FOI3        
        CALL    BIP    ;BIPA UMA VEZ
        BCF     LED    ; APAGA LED
        MOVLW   .1     ;SALVA NA EEPROM QUE FOI DISPARADO
        MOVWF   TMP
        MOVLW   END_DISP
        CALL    EEWRITE
        GOTO    VOLTA3

PULAT1
 		MOVLW 	TEMPO_S_TEC 	;CARGA DE TEMPO (SAI SE NO HOUVER DIGITAO)
        MOVWF    Z1
        CLRF     Z0
		MOVLW   .8
        MOVWF   CNT1
LOOP4
        CALL    TECLANDO
        MOVWF   TEMP
        XORLW   .4          ;TODOS OS BIT OBTIDOS
        BTFSC   STATUS,Z
        GOTO    COMPARA
 		MOVFW   TEMP
        XORLW   .2          ;TODOS OS BIT OBTIDOS
        BTFSS   STATUS,Z
        GOTO    MAIN
        BTFSS   BOTAO_1     ;AGUARDA SOLTAR BOTO
        GOTO    $-1

	    MOVLW   .1
        CALL    DELAYM
        GOTO    MAIN

COMPARA
	  MOVLW     SENHA_TROCA    ;TESTA SE  SENHA COMANDO
      XORWF   	BITSENHA,W
      BTFSC   	STATUS,Z
      GOTO      TROCANDO_SENHA
    
      MOVLW   	.0            ;TESTA SE ACERTOU A SENHA GRAVADA
      CALL    	EEREAD
      XORWF   	BITSENHA,W
      BTFSS   	STATUS,Z
      GOTO      ERRO
;==================================================================
;               SENHA CORRETA
;==================================================================
      BSF     	RELE 			;LIGA REL PARA PERMITIR FUNCIONAMENTO 
      BSF       LED             ;ACENDE LED 
	  BCF       T0IE_           ;DESLIGA INTERRUPO DO TIMER 0
      MOVLW     .0
      MOVWF     TMP
      MOVLW     END_BLOQUEADO
      CALL      EEWRITE
      MOVLW     .0
      MOVWF     TMP
      MOVLW     END_DISP        ;DESLIGA O DISPARO NA EEPROM
      CALL      EEWRITE
      CALL      APITO_C
LIBERA
      MOVLW   VALOR_X2          ;PREPARA CONTADORES PARA UM NOVO TEMPO
      MOVWF   X2
      CLRF    X1

      MOVLW     .1               ;SALVA NA EEPROM
      MOVWF     TMP
      MOVLW     END_SIT
      CALL      EEWRITE         ;MUDA SITUAO NA EEPROM QUANTO AO REL   

      MOVLW   VALOR_X2  ;VALOR DO TEMPO PARA DESLIGAMENTO
      MOVWF   TMP
      MOVLW   END_TIME ;APONTA ENDEREO DA EEPROM 
      CALL    EEWRITE  ;RECUPERA TEMPO DE DESLIGAMENTO 

      MOVLW  	NUMERO_DE_ERROS  ;CARREGA CONTADOR DE MAX. DE SENHAS ERRADAS
	  MOVWF 	RER 
      BCF       FLAG_DIGITOU     ;APAGA FLAG
      GOTO      MAIN


;============================================================================
;              SENHA INCORRETA
;============================================================================
ERRO
      BCF       FLAG_DIGITOU
      MOVLW     .5
      MOVWF     CNT
LOOP2
      CALL      BIP   ;PRODUZ UM NICO BIP POUCO MAIS LONGO
	  CALL      BIP
	  CALL      BIP
      CALL      BIP
	  CALL      BIP
      MOVLW     .2
      CALL      DELAYM
      DECFSZ    CNT,F
      GOTO      LOOP2
      DECFSZ    RER,F  ;TESTA NMERO MXIMO DE TENTATIVAS ERRADAS
      GOTO      MAIN   ; AINDA NO ULTRAPASSOU LIMITE
      MOVLW     .1     ;ULTRAPASSOU: TRAVAR TECLADO E BLOQUEAR TAMBM
      MOVWF     TMP
      MOVLW     END_BLOQUEADO  ;SALVA NA EEPROM
      CALL      EEWRITE
      MOVLW     .1
      MOVWF     TMP
      MOVLW     END_TECLADO_TRAVADO ;SALVA NA EEPROM
      CALL      EEWRITE

TECLADO_TRAVADO
      MOVLW  	NUMERO_DE_ERROS  ;CARREGA CONTADOR DE MAX. DE SENHAS ERRADAS
	  MOVWF 	RER 
      MOVLW     T_TRAVADO
      MOVWF     CNT
LOOP3


;------------------------------OPO POR DEFINE---------------------------------------
  ;BIPA 5 VEZES AO LIGAR CONTATO COM SISTEMA BLOQUEADO
 IFDEF TRAVA_ENTRADA_SENHA
      BSF       LED ; PISCA RPIDO JUNTO COM O BIP
      CALL      BIP
      BCF       LED
      MOVLW     .1
      CALL      DELAYM ;DELAY ENTRE BIPS
      BSF       LED
      CALL      BIP
      BCF       LED
      MOVLW     .1
      CALL      DELAYM
      BSF       LED
      CALL      BIP
      BCF       LED
      MOVLW     .15
      CALL      DELAYM
      DECFSZ    CNT,F ;AGUARDA TERMINO DO PERODO
      GOTO      LOOP3 ;RETORNA AO LOOP
      MOVLW     .0
      MOVWF     TMP
      MOVLW     END_TECLADO_TRAVADO
      CALL      EEWRITE
      GOTO      MAIN  ; VAI PARA O COMEO DO PROGRAMA
  ENDIF
;---------------------------------------------------------------------------------

CONT_BLOQUEADO

;--------------------------OPO POR DEFINE---------------------------------------
; PERMITE LIGAR RELE POR APENAS 2 SEGUNDOS AO LIGAR CHAVE DE CONTATO QUANDO BLOQUEADO
; (PARA SIMULAR FALHA)
; O LED ACENDE POR 2 SEGUNDOS E DEPOIS COMEA A PISCAR   
    IFDEF  LIGA_MOMENTANEO
      BSF       LED  
      BSF       RELE   ; LIGA MOMENTANEAMENTE PARA DAR IMPRESSO DE FALHA NO MOTOR
      MOVLW     TEMPO_SIMULACAO ;TEMPO QUE PERMITIR LIGAR O RELE (BREVEMENTE)
      MOVWF     Y1

RET99
      MOVLW     .20   ;PARA 1 SEGUNDOS
      CALL      DELAYM ;ROTINA DE 50 MS X20
      DECFSZ    Y1,F
      GOTO      RET99

    ENDIF
;----------------------------------------------------------------------------------   
      BCF       RELE  ;REL DESLIGADO
      BCF       LED   ;LED DESLIGADO
      GOTO      MAIN

;============================================================================
;              TROCANDO SENHA DO USURIO
;============================================================================
TROCANDO_SENHA
        CALL    APITO_C
	    MOVLW   .8
        MOVWF   CNT1
        CALL    TECLANDO
		XORLW   .2          ;TODOS OS BIT OBTIDOS
        BTFSS   STATUS,Z
        GOTO    TT0
SAI_T
        BTFSS   GPIO,1
        GOTO    $-1
	    MOVLW   .5
        CALL    DELAYM
        CALL    BIP
		BCF       FLAG_DIGITOU
        GOTO    MAIN

TT0
		MOVLW   .0
        CALL    EEREAD         ;PEGA SENHA NA EPROM
        XORWF   BITSENHA,W
        BTFSS   STATUS,Z
        GOTO    ERRO
		CALL    APITO_C       ;2 APITOS CURTOS
        MOVLW   .3
        CALL    DELAYM
        CALL    APITO_C
;==========================================================================
;         DIGITANDO A NOVA SENHA
;==========================================================================
	    MOVLW   .8
        MOVWF   CNT1
        CALL    TECLANDO	  ;OBTM A NOVA SENHA
		XORLW   .2          ;TODOS OS BIT OBTIDOS
        BTFSS   STATUS,Z
        GOTO    TT1
        GOTO    SAI_T
TT1
        MOVFW   BITSENHA
        MOVWF   TEMP2		  ;SALVA SENHA NOVA PARA COMPARAO
       	CALL    APITO_C       ;3 APITOS CURTOS
        MOVLW   .3
        CALL    DELAYM
       	CALL    APITO_C       
        MOVLW   .3
        CALL    DELAYM
        CALL    APITO_C
;==========================================================================
;         CONFIRMANDO A NOVA SENHA
;==========================================================================
	    MOVLW   .8
        MOVWF   CNT1
        CALL    TECLANDO    ;OBTEM CONFIRMAO DA SENHA NOVA
        XORLW   .2          ;TODOS OS BIT OBTIDOS
        BTFSS   STATUS,Z
        GOTO    TT2
        GOTO    SAI_T
TT2
        MOVFW   TEMP2
        XORWF   BITSENHA,W  ;COMPARA SENHA NOVA COM A CONFIRMAO DELA
		BTFSC   STATUS,Z
        GOTO    ERRO
        MOVFW   BITSENHA	;ESCREVE NOVA SENHA NA EEPROM
        MOVWF   TMP
        MOVLW   .0 
        CALL    EEWRITE
        MOVLW   END_DISP
        CALL    EEREAD
        BTFSS   STATUS,Z
        BCF     LED
        BTFSC   STATUS,Z
        BSF     LED

        BCF     FLAG_DIGITOU
        CALL    APITO_L  ;APITO LONGO
        GOTO    MAIN
;==========================================================================
;         ROTINA DE OBTENO DA SENHA
;==========================================================================
TECLANDO
        MOVLW   TEMPO_S_TEC  ;CARGA DO CONTADOR Z1/Z0
        MOVWF   Z1
        CLRF    Z0
L_TECLANDO
TX3
ZER_CONT
        
        MOVLW  DEBOUNCE      ;CARGA PARA DEBOUNCE DE INTERRUPTORES
        MOVWF  CNT 
        BTFSC  BOTAO_0 
        GOTO   OUTRO_BOT
RET_DEB
        GOTO   $+1
        GOTO   $+1
		GOTO   $+1
        GOTO   $+1
        BTFSC  BOTAO_0 
        GOTO   ZER_CONT
        BCF    LED      ;DESLIGA LED CASO ESTEJA LIGADO PELO PISCA
        DECFSZ CNT,F
        GOTO   RET_DEB
        MOVLW  DEBOUNCE 
        MOVWF  CNT
RET_DEBS
        GOTO   $+1
        GOTO   $+1
		GOTO   $+1
        GOTO   $+1
        BTFSS  BOTAO_0 
        GOTO   $-1
		DECFSZ CNT,F
        GOTO   RET_DEBS
        BSF    STATUS,C
        GOTO   PEGA

OUTRO_BOT
ZER_CONT1
        MOVLW   T_APERTADO
        MOVWF   TIMER1
        MOVLW  	DEBOUNCE 
        MOVWF  	CNT
        BTFSC  	BOTAO_1
        GOTO    TEMPO_SEM_APERTAR
        BCF    LED      ;DESLIGA LED CASO ESTEJA LIGADO PELO PISCA
RET_DEB1
        GOTO   	$+1
        GOTO   	$+1
		GOTO   	$+1
        GOTO   	$+1
        BTFSC  BOTAO_1
        GOTO   	ZER_CONT1
        DECFSZ 	CNT,F
        GOTO   	RET_DEB1
        MOVLW  DEBOUNCE 
        MOVWF  CNT
RET_DEBS1
        GOTO   $+1
        GOTO   $+1
		GOTO   $+1
        GOTO   $+1
        BTFSS  BOTAO_1
        GOTO   LOOP1A_INC
        DECFSZ 	CNT,F
        GOTO   	RET_DEBS1
        BCF    	STATUS,C
        GOTO   	PEGA
LOOP1A_INC
        CALL    AGUARDE
        DECFSZ   TIMER1,F
        GOTO    RET_DEBS1
        CALL    BIP
        BCF     FLAG_DIGITOU
        RETLW   .2

PEGA
       BSF      FLAG_DIGITOU
       RLF   	BITSENHA,F
       DECFSZ  	CNT1,F
       GOTO  	BIPAR
       RETLW    .4          ;SENHA COMPLETA

BIPAR
	   CALL   	BIP
       GOTO     TECLANDO
TEMPO_SEM_APERTAR
       BTFSS    FLAG_DIGITOU
       RETLW    0XFF
       MOVLW    .146
       ADDLW    .1
       BTFSS    STATUS,Z
       GOTO     $-2
       DECFSZ   Z0,F
       GOTO     L_TECLANDO
       DECFSZ   Z1,F
       GOTO     L_TECLANDO
       BCF      FLAG_DIGITOU
	   MOVLW 	TEMPO_S_TEC 
       MOVWF    Z1
       CALL     APITO_C
       CLRF     BITSENHA
       RETLW    .2
    
;=================================================================================
;				GERA BIP AUDVEL NA DIGITAO
;=================================================================================
;GERA BIP 
BIP
    MOVLW 	.200
	MOVWF  	TEMP2
       
CONT6
	BCF 	SOM
	MOVLW 	0X30
	CALL 	WAIT_US
	BSF 	SOM
	MOVLW 0X30
	CALL 	WAIT_US
	DECFSZ  TEMP2,F
	GOTO  	CONT6
   	RETURN
WAIT_US 
	MOVWF 	XLC
	DECFSZ 	XLC, F
	GOTO    $-1	
	RETURN	
;============================================================================================
;                                  APITOS
;============================================================================================
APITO_L
      MOVLW     0X30			;LONGO
      GOTO      AP1
APITO_C

      MOVLW     0X10			;BREVE
AP1
      MOVWF     CNT
      CALL      BIP
      DECFSZ    CNT,F
      GOTO      $-2
      RETURN




;==================================================================
;                      ROTINAS DE EEPROM
;==================================================================
DELAYM     ; DELAY= 50 milisegundos X 'W'
   MOVWF  R0
XC
   CALL  AGUARDE
   DECFSZ R0,F
   GOTO XC
   RETURN

AGUARDE		; ROTINA DE ATRASO ENTRE OPERAES DE EEPROM (50 MSEG)
	MOVLW  .50
	MOVWF	R2
    CALL DELAY_1MS  ;ROTINA DE 1 MILISEGUNDOS
    DECFSZ  R2,F
    GOTO  $-2
    RETURN
DELAY_1MS
	MOVLW	.199
	MOVWF	R1
    CLRWDT
    NOP
	DECFSZ	R1,F
	GOTO $-3
	RETURN

;============================================================================================
;                         ESCREVER  NA    EEPROM
;============================================================================================
EEWRITE

;ENDEREO EM 'W'
;DADOS A ESCREVER ESTA EM TMP


	BANK1
	movwf EEADR	
	BANK0
	MOVFW  TMP		;PEGA PRIMEIRO DADO
	BANK1
	movwf EEDATA
	bcf EECON1, EEIF	
	bsf EECON1, WREN ; enable Write\par
	movlw 0x55	
	movwf EECON2	
	movlw 0xAA	
	movwf EECON2	
	bsf EECON1, WR	
WRITE_SN_A 	
	btfsc EECON1, WR ; Write complete ?\par
	goto WRITE_SN_A	
	bcf EECON1, WREN ; disable Write\par
 	BANK0
	
ESPERA1			;DELAY ENTRE APAGAMENTOS
	CALL  AGUARDE
	RETURN

;============================================================================================
;                             LER  A   EEPROM
;============================================================================================

EEREAD
;endereo esta em ADDRESS
;DADOS LIDOS SERO ESCRITOS EM TMP0
        
	BANK1
	movwf EEADR	
	bsf EECON1, RD ; do a read\par
    btfsc EECON1, RD ; Read done ?\par
	goto $-1	
	movf EEDATA,W
	BANK0
	MOVWF	TMP	;RECUPERA PRIMEIRO NUMERO DA EEPROM
	RETURN
;============================================================================================
;                                PR-GRAVAO DA  EEPROM
;============================================================================================
    ORG 0X2100   
    DE  SENHA_INIT    ;SENHA INICIAL

    ORG 0X2100  + END_SIT
    DE  .1            ;REL LIGADO

    ORG 0X2100  + END_BLOQUEADO
    DE  .0            ;DESBLOQUEADO

    ORG 0X2100  + END_DISP
    DE  .0            ;SEM SER DISPARADO

    ORG 0X2100  + END_TIME
    DE  VALOR_X2            ;SEM SER DISPARADO

    ORG 0X2100  + END_TECLADO_TRAVADO
    DE  .0           ;TECLADO DESTRAVADO



       END
